
import base_section

from runtest import conditions


DEFAULT_HEAT_PLUGIN_PARAMETERS = {
    'auth_version': 3
}


class HeatPlugin(base_section.BaseSection):

    name = "heat_plugin"
    options = [
        'admin_password',
        'admin_project_name',
        'admin_username',
        'auth_url',
        'auth_version',
        'boot_config_env',
        'build_interval',
        'build_timeout',
        'ca_file',
        'catalog_type',
        'connectivity_timeout',
        'convergence_engine_enabled',
        'disable_ssl_certificate_validation',
        'fixed_network_name',
        'fixed_subnet_name',
        'floating_network_name',
        'heat_config_notify_script',
        'image_ref',
        'instance_type',
        'ip_version_for_ssh',
        'keypair_name',
        'minimal_image_ref',
        'minimal_instance_type',
        'network_for_ssh',
        'password',
        'project_domain_id',
        'project_domain_name',
        'project_name',
        'region',
        'sighup_config_edit_retries',
        'sighup_timeout',
        'skip_functional_test_list',
        'skip_functional_tests',
        'skip_scenario_test_list',
        'skip_scenario_tests',
        'skip_test_stack_action_list',
        'ssh_channel_timeout',
        'ssh_timeout',
        'tenant_network_mask_bits',
        'user_domain_id',
        'user_domain_name',
        'username',
        'volume_size',
        'vm_to_heat_api_insecure',
    ]

    @property
    def admin_password(self):
        c = conditions.BaseRule('keystone.server.enabled', 'eq', True)
        return self.get_item_when_condition_match(
            'keystone.server.admin_password', c)

    @property
    def admin_project_name(self):
        pass

    @property
    def admin_username(self):
        c = conditions.BaseRule('keystone.server.enabled', 'eq', True)
        return self.get_item_when_condition_match(
            'keystone.server.admin_name', c)

    @property
    def auth_url(self):
        c = conditions.BaseRule('keystone.server.enabled', 'eq', True)
        protocol = self.get_item_when_condition_match(
            'keystone.server.bind.private_protocol', c) or 'http'
        vip = self.get_item_when_condition_match(
            '_param.cluster_vip_address', c)
        port = self.get_item_when_condition_match(
            'keystone.server.bind.private_port', c)
        return "{}://{}:{}".format(protocol, vip, port)

    @property
    def auth_version(self):
        admin_identity = \
            'keystone.client.os_client_config.cfgs.root.content.clouds.admin_identity.{}'
        c = conditions.BaseRule('keystone.client.enabled', 'eq', True)
        AUTH_VERSION = self.get_item_when_condition_match(
            admin_identity.format('identity_api_version'), c)
        if AUTH_VERSION:
            return AUTH_VERSION
        else:
            return DEFAULT_HEAT_PLUGIN_PARAMETERS['auth_version']

    @property
    def boot_config_env(self):
        pass

    @property
    def build_interval(self):
        pass

    @property
    def build_timeout(self):
        pass

    @property
    def ca_file(self):
        pass

    @property
    def catalog_type(self):
        pass

    @property
    def connectivity_timeout(self):
        pass

    @property
    def convergence_engine_enabled(self):
        pass

    @property
    def disable_ssl_certificate_validation(self):
        pass

    @property
    def fixed_network_name(self):
        pass

    @property
    def fixed_subnet_name(self):
        pass

    @property
    def floating_network_name(self):
        c = conditions.BaseRule('neutron.client.enabled', 'eq', True)
        networks_legacy = self.get_item_when_condition_match(
            'neutron.client.server.admin_identity.network', c) or {}
        networks = self.get_item_when_condition_match(
            'neutron.client.resources.v2.admin_identity.network', c) or {}
        networks.update(networks_legacy)
        for network_name, network in networks.items():
            if network.get('router_external') is True:
                return network_name

    @property
    def heat_config_notify_script(self):
        pass

    @property
    def image_ref(self):
        pass

    @property
    def instance_type(self):
        c = conditions.BaseRule(field='keystone.client.enabled', op='eq',
                                val=True)
        nodes = self.get_nodes_where_condition_match(c)
        keystone_profile_admin = self.runtest_opts.get(
            'keystone_profile_admin', {})
        res = self.authenticated_openstack_module_call(
            nodes[0],
            'novang.flavor_list',
            profile=keystone_profile_admin)[nodes[0]]
        try:
            # Need to have > 3 GB disk, but baremetal deployments
            # can have 120 GB disk flavor, so better to select < 10 Gb.
            flavors = [f for f in res
                       if (res[f]['disk'] > 3) and (res[f]['disk'] < 10)]
        except (ValueError, TypeError, KeyError):
            flavors = None
        if flavors:
            return flavors[0]
        else:
            return

    @property
    def ip_version_for_ssh(self):
        pass

    @property
    def keypair_name(self):
        pass

    @property
    def minimal_image_ref(self):
        pass

    @property
    def minimal_instance_type(self):
        c = conditions.BaseRule('nova.client.enabled', 'eq', True)
        flavors = self.get_item_when_condition_match(
            'nova.client.server.admin_identity.flavor', c)
        if not flavors:
            return
        minimal_instance_type_name, _ = sorted(
            flavors.items(),
            key=lambda (k, v): (v, k)
        )[0]
        return minimal_instance_type_name

    @property
    def network_for_ssh(self):
        pass

    @property
    def password(self):
        c = conditions.BaseRule('keystone.server.enabled', 'eq', True)
        return self.get_item_when_condition_match(
            'keystone.server.admin_password', c)

    @property
    def project_domain_id(self):
        pass

    @property
    def project_domain_name(self):
        pass

    @property
    def project_name(self):
        c = conditions.BaseRule('keystone.server.enabled', 'eq', True)
        return self.get_item_when_condition_match(
            'keystone.server.admin_tenant', c)

    @property
    def region(self):
        c = conditions.BaseRule('keystone.server.enabled', 'eq', True)
        return self.get_item_when_condition_match(
            'keystone.server.region', c)

    @property
    def sighup_config_edit_retries(self):
        pass

    @property
    def sighup_timeout(self):
        pass

    @property
    def skip_functional_test_list(self):
        skip_list = []

        # Update of allocation_pools is not allowed in case of Contrail
        contrail_enabled = conditions.BaseRule('*.opencontrail.control.enabled',
                                               'eq', True, multiple='any')
        if contrail_enabled.check(self.pillar):
            skip_list.append('UpdateSubnetTest')

        # UpdateServerNetworksTest, StackTemplateValidateTest : These tests are
        # supported since Queens.
        pike = conditions.BaseRule('*.heat.server.version', 'eq', 'pike')
        if pike.check(self.pillar):
            skip_list.extend(['UpdateServerNetworksTest',
                              'StackTemplateValidateTest'])

        if not skip_list:
            pass
        else:
            skip_tests = ', '.join(skip_list)
            return skip_tests

    @property
    def skip_functional_tests(self):
        pass

    @property
    def skip_scenario_test_list(self):
        pass

    @property
    def skip_scenario_tests(self):
        pass

    @property
    def skip_test_stack_action_list(self):
        pass

    @property
    def ssh_channel_timeout(self):
        pass

    @property
    def ssh_timeout(self):
        pass

    @property
    def tenant_network_mask_bits(self):
        pass

    @property
    def user_domain_id(self):
        pass

    @property
    def user_domain_name(self):
        pass

    @property
    def username(self):
        c = conditions.BaseRule('keystone.server.enabled', 'eq', True)
        return self.get_item_when_condition_match(
            'keystone.server.admin_name', c)

    @property
    def volume_size(self):
        pass

    @property
    def vm_to_heat_api_insecure(self):
        c = conditions.BaseRule('keystone.client.enabled', 'eq', True)
        api_v3_enabled = self.get_item_when_condition_match(
            'keystone.client.resources.v3.enabled', c)
        if not api_v3_enabled:
            if self.get_item_when_condition_match(
                    'keystone.client.server.identity.service.heat.endpoints.public_protocol', c) == 'https':
                return True
        if self.get_item_when_condition_match(
                'keystone.client.resources.v3.services.heat.endpoints.heat_public_v1.url', c).startswith('https'):
            return True
